import { ArkaneConnect } from "@arkane-network/arkane-connect";
import { PartialTxParams } from "@0x/subproviders";
import { BaseWalletSubprovider } from "@0x/subproviders/lib/src/subproviders/base_wallet_subprovider";
import { ArkaneSubProviderOptions } from "./index";
import { AuthenticationOptions, AuthenticationResult } from '@arkane-network/arkane-connect/dist/src/connect/connect';
import { Network } from "@arkane-network/arkane-connect/dist/src/models/Network";
import { Account } from '@arkane-network/arkane-connect/dist/src/models/Account';
export declare class ArkaneWalletSubProvider extends BaseWalletSubprovider {
    readonly arkaneConnect: ArkaneConnect;
    private wallets;
    network?: Network;
    private options;
    private authenticated;
    private lastWalletsFetch?;
    constructor(options: ArkaneSubProviderOptions);
    startGetAccountFlow(authenticationOptions?: AuthenticationOptions): Promise<Account | {}>;
    private refreshWalletsFromApi;
    /**
     * Retrieve the accounts associated with the eth-lightwallet instance.
     * This method is implicitly called when issuing a `eth_accounts` JSON RPC request
     * via your providerEngine instance.
     *
     * @return An array of accounts
     */
    getAccountsAsync(): Promise<string[]>;
    private shouldRefreshWallets;
    checkAuthenticated(): Promise<AuthenticationResult>;
    /**
     * Signs a transaction with the account specificed by the `from` field in txParams.
     * If you've added this Subprovider to your app's provider, you can simply send
     * an `eth_sendTransaction` JSON RPC request, and this method will be called auto-magically.
     * If you are not using this via a ProviderEngine instance, you can call it directly.
     * @param txParams Parameters of the transaction to sign
     * @return Signed transaction hex string
     */
    signTransactionAsync(txParams: PartialTxParams): Promise<string>;
    private constructEthereumTransationSignatureRequest;
    /**
     * Sign a personal Ethereum signed message. The signing account will be the account
     * associated with the provided address.
     * If you've added this Subprovider to your app's provider, you can simply send an `eth_sign`
     * or `personal_sign` JSON RPC request, and this method will be called auto-magically.
     * If you are not using this via a ProviderEngine instance, you can call it directly.
     * @param data Hex string message to sign
     * @param address Address of the account to sign with
     * @return Signature hex string (order: rsv)
     */
    signPersonalMessageAsync(data: string, address: string): Promise<string>;
    /**
     * Sign an EIP712 Typed Data message. The signing address will associated with the provided address.
     * If you've added this Subprovider to your app's provider, you can simply send an `eth_signTypedData`
     * JSON RPC request, and this method will be called auto-magically.
     * If you are not using this via a ProviderEngine instance, you can call it directly.
     * @param address Address of the account to sign with
     * @param data the typed data object
     * @return Signature hex string (order: rsv)
     */
    signTypedDataAsync(address: string, typedData: any): Promise<string>;
    private getWalletIdFrom;
}
