"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Arkane = void 0;
var tslib_1 = require("tslib");
var ArkaneWalletSubProvider_1 = require("./ArkaneWalletSubProvider");
var NonceTracker_1 = require("./NonceTracker");
var SignedVersionedTypedDataSubProvider_1 = require("./SignedVersionedTypedDataSubProvider");
var arkane_connect_1 = require("@arkane-network/arkane-connect");
var ProviderEngine = require('@arkane-network/web3-provider-engine');
var CacheSubprovider = require('@arkane-network/web3-provider-engine/subproviders/cache');
var FixtureSubprovider = require('@arkane-network/web3-provider-engine/subproviders/fixture');
var FilterSubprovider = require('@arkane-network/web3-provider-engine/subproviders/filters');
var RpcSubprovider = require('@arkane-network/web3-provider-engine/subproviders/rpc');
var SubscriptionsSubprovider = require('@arkane-network/web3-provider-engine/subproviders/subscriptions');
var SanitizingSubprovider = require('@arkane-network/web3-provider-engine/subproviders/sanitizer');
var InflightCacheSubprovider = require('@arkane-network/web3-provider-engine/subproviders/inflight-cache');
var WebsocketSubprovider = require('@arkane-network/web3-provider-engine/subproviders/websocket');
var ArkaneSubProvider = /** @class */ (function () {
    function ArkaneSubProvider() {
    }
    ArkaneSubProvider.prototype.arkaneConnect = function (network) {
        this.network = network;
        this.originalNetwork = network;
        return this.ac;
    };
    ArkaneSubProvider.prototype.changeNetwork = function (network) {
        if (network && network.nodeUrl) {
            this.network = network;
            this.nonceSubProvider.rpcUrl = network.nodeUrl;
            this.rpcSubprovider.rpcUrl = network.nodeUrl;
            this.arkaneSubProvider.network = network;
        }
        else {
            console.warn("Not changing to network, not sufficient data: resetting network", network);
            this.resetNetwork();
        }
    };
    ArkaneSubProvider.prototype.checkAuthenticated = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                if (!this.arkaneSubProvider) {
                    throw new Error("Please initialise provider first (Arkane.createArkaneProviderEngine)");
                }
                return [2 /*return*/, this.arkaneSubProvider.checkAuthenticated()];
            });
        });
    };
    ArkaneSubProvider.prototype.authenticate = function (authenticationOptions) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                if (!this.arkaneSubProvider) {
                    throw new Error("Please initialise provider first (Arkane.createArkaneProviderEngine)");
                }
                return [2 /*return*/, this.arkaneSubProvider.startGetAccountFlow(authenticationOptions)];
            });
        });
    };
    ArkaneSubProvider.prototype.resetNetwork = function () {
        this.network = this.originalNetwork;
    };
    ArkaneSubProvider.prototype.createArkaneProviderEngine = function (options) {
        var _this = this;
        var connectionDetails = this.getConnectionDetails(options);
        var engine = new ProviderEngine({ pollingInterval: options.pollingInterval || 150000 });
        engine.addProvider(new FixtureSubprovider({
            web3_clientVersion: 'ArkaneProviderEngine/v0.0.1/javascript',
            net_listening: true,
            eth_hashrate: '0x00',
            eth_mining: false,
            eth_syncing: true,
        }));
        this.nonceSubProvider = new NonceTracker_1.NonceTrackerSubprovider({ rpcUrl: connectionDetails.endpointHttpUrl });
        engine.addProvider(this.nonceSubProvider);
        engine.addProvider(new SanitizingSubprovider());
        engine.addProvider(new CacheSubprovider());
        engine.addProvider(new InflightCacheSubprovider());
        if (!connectionDetails.endpointWsUrl) {
            engine.addProvider(new SubscriptionsSubprovider());
            engine.addProvider(new FilterSubprovider());
        }
        else {
            engine.addProvider(new WebsocketSubprovider({ rpcUrl: connectionDetails.endpointWsUrl }));
        }
        this.arkaneSubProvider = new ArkaneWalletSubProvider_1.ArkaneWalletSubProvider(options);
        this.ac = this.arkaneSubProvider.arkaneConnect;
        this.signedVersionedTypedDataSubProvider = new SignedVersionedTypedDataSubProvider_1.SignedVersionedTypedDataSubProvider(this.arkaneSubProvider);
        engine.addProvider(this.signedVersionedTypedDataSubProvider);
        this.rpcSubprovider = new RpcSubprovider({ rpcUrl: connectionDetails.endpointHttpUrl });
        return options.skipAuthentication
            ? Promise.resolve(this.startEngine(engine))
            : this.arkaneSubProvider.getAccountsAsync().then(function () { return _this.startEngine(engine); });
    };
    ArkaneSubProvider.prototype.getConnectionDetails = function (options) {
        if (options.network && options.network.nodeUrl) {
            return {
                endpointHttpUrl: options.network.nodeUrl
            };
        }
        var secretType = options.secretType ? options.secretType : arkane_connect_1.SecretType.ETHEREUM;
        var endpoint = secretType.toLowerCase() + "-node" + (options.environment && !options.environment.startsWith('prod') ? '-' + options.environment : '') + ".arkane.network";
        if (secretType == arkane_connect_1.SecretType.ETHEREUM || secretType == arkane_connect_1.SecretType.MATIC) {
            return {
                endpointHttpUrl: 'https://' + endpoint,
                endpointWsUrl: 'wss://ws-' + endpoint
            };
        }
        return {
            endpointHttpUrl: 'https://' + endpoint
        };
    };
    ArkaneSubProvider.prototype.startEngine = function (engine) {
        engine.addProvider(this.arkaneSubProvider);
        engine.addProvider(this.rpcSubprovider);
        // network connectivity error
        engine.on('error', function (err) {
            // report connectivity errors
            console.error(err.stack);
        });
        // start polling for blocks
        engine.start();
        return engine;
    };
    return ArkaneSubProvider;
}());
var ConnectionDetails = /** @class */ (function () {
    function ConnectionDetails(endpointHttpUrl, endpointWsUrl) {
        this.endpointHttpUrl = endpointHttpUrl;
        this.endpointWsUrl = endpointWsUrl;
    }
    return ConnectionDetails;
}());
if (typeof window !== 'undefined') {
    window.Arkane = new ArkaneSubProvider();
}
exports.Arkane = ArkaneSubProvider.prototype;
//# sourceMappingURL=index.js.map